/*!
 * imageBW.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Example game script for simple image loading. Converts to black and white
 *
 * Note currently resizing the browser window does NOT change the canvas size
 * So... probably want a boundary edge displayed on the canvas (via html style)
 */
 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theProgram (singleton) Object                            theProgram Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theProgram = 
{
    // ------------------------------------------------------------------------
    // Pseudo-constants
    // ------------------------------------------------------------------------
    IMAGE_CANVAS_ID: "mainImageCanvas",     // canvas id used in html
    
    // ------------------------------------------------------------------------
    // Variables
    // ------------------------------------------------------------------------
    width:          400,           // canvas width... likely will be reset
    height:         400,           // canvas height... likely will be reset
    startTime:      0,             
    endTime:        0,             
    delay:          0.01,
    imageObj:       null,
    
    xOffset:        5,
    yOffset:        5,
    
    // ------------------------------------------------------------------------
    // Functions
    // ------------------------------------------------------------------------
    //                                                          Init
    // ------------------------------------------------------------------------
    Init: function() 
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
		canvas.addEventListener('click', theProgram.onClick, false);
		canvas.addEventListener('touch', theProgram.onClick, false);
        
        this.imageObj = new Image();
        this.imageObj.onload = function() 
                                        {
                                            theProgram.drawOrigImage();
                                        };
        this.imageObj.src = 'hotair.png';
    },
    
    // ------------------------------------------------------------------------
    // EventHandler                                             onClick
    // ------------------------------------------------------------------------
    onClick: function(e)
    {
        theProgram.drawOrigImage();
        theProgram.drawBWimage();
    },
    
    // ------------------------------------------------------------------------
    // drawOrigImage
    // ------------------------------------------------------------------------
    drawOrigImage: function()
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
        var ctx = canvas.getContext('2d');
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        
        ctx.drawImage(this.imageObj, this.xOffset, this.yOffset);

        ctx.font="16px Georgia";
        ctx.fillText("Click/tap to see the", this.imageObj.width+20, 80);
        ctx.fillText("black and white image", this.imageObj.width+20, 100);
    },
    

    // ------------------------------------------------------------------------
    // drawBWimage  -- must call drawOrigImage before this
    // ------------------------------------------------------------------------
    drawBWimage: function()
    {
        var canvas = document.getElementById(this.IMAGE_CANVAS_ID);
        var ctx = canvas.getContext('2d');

        // Get the image data from the canvas context
        var imageData = ctx.getImageData(this.xOffset, this.yOffset, 
                                         this.imageObj.width, this.imageObj.height);
        var data = imageData.data;

        // This gives us the image data in RGBA format - one byte for each
        // So to iterate over all pixels we would do:
        //for(var i = 0, n = data.length; i < n; i += 4) 
        // {
            // var red = data[i];
            // var green = data[i + 1];
            // var blue = data[i + 2];
            // var alpha = data[i + 3];
        // }
        
        // We must 'create' a new image
        var BWimageData = ctx.createImageData(this.imageObj.width, this.imageObj.height);
        //var BWimageData = ctx.createImageData(this.imageObj);

        // Let's time how long it takes
        var dateObj  = new Date();
        this.startTime = dateObj.getTime();
        
        var avgColor = 0;
        console.log("BWimageData.data.length = " + BWimageData.data.length);
        for (var pixelIndex=0; pixelIndex < BWimageData.data.length; pixelIndex += 4)
        {
            //console.log("---" + avgColor);
            // Apply luminance algorithm of: 0.3*red + 0.59*green + 0.11*blue
            avgColor = Math.round( 0.3*data[pixelIndex] + 0.59*data[pixelIndex +1] + 0.11*data[pixelIndex +2]);
            BWimageData.data[pixelIndex    ] = avgColor;  // red   color
            BWimageData.data[pixelIndex + 1] = avgColor;  // green color
            BWimageData.data[pixelIndex + 2] = avgColor;  // blue  color
            BWimageData.data[pixelIndex + 3] = 255;       // 0 = fully transparent, 255 = no transparency
        }
        
        // Want to draw the black and white image to the right of original
        // Offset gives spacing between
        var BWimageX = 2*this.xOffset + this.imageObj.width;
        var BWimageY = this.yOffset;
        ctx.putImageData(BWimageData, BWimageX, BWimageY);
        
        // end timing
        this.endTime = dateObj.getTime();
        
        var totalTime = this.endTime - this.startTime;
        console.log("Time to convert and display = " + totalTime + " ms");
    },
    
};  // end theProgram variable

// ----------------------------------------------------------------------------
//                                                          window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function()
{
    //console.log("DEBUG: window.onload entered");

    // Initialize and Start the game
    theProgram.Init();
    
	// Run the program (start the program loop)
	// runMain(); not needed here
};

// ----------------------------------------------------------------------------
//                                                    the main program 'loop'
// Event handlers/listeners cover the game input
// ----------------------------------------------------------------------------
function runMain()
{
    // Be certain to call runGame again	
	requestAnimationFrame(runMain);
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file
